function klm = Peanoshfun(locpdeg, gvidx)
% return the exponents of the barycentric coordinates which define the Peano shape functions
% INPUT:
%  locpdeg ... 1-by-4 vector of polynomial degrees in the element
%          locpdeg(1) degree of the interior;  locpdeg(2:4) degrees of the edges
%  gvidx ... 1-by-3 vector of the global indices of vertices (to determin the orientation)
%%  idx ... 1-by-7 vector of global indices (to the stiffness matrix)
%%          of the triangle's DOFs:
%%          idx(1:3)==vertex DOFs; idx(4:6)==the first edge DOFs; idx(7)==the first bubble DOF;
%%         (needed because of the orientation of edges and boundary conditions)
% OUTPUT:
%  klm ... NK-by-3 matrix of exponents: klm(ishf,1)==k, klm(ishf,2)==l, klm(ishf,3)==m
%          shfun(ishf) = lmb_1^k lmb_2^l lmb_3^m,
%          where lmb_i is the barycentric coordinate corresponding to the i-th vertex
%          ishf = local index of the shape functions
%  klm SHOULD BE RENAMED TO lmn
%
% The set of Peano shape functin is defined as follows
% (a) vertex functions
%     phi_i = lmb_i,  i=1,2,3
% (b) edge functions - edges e1,e2,e3 have polynomial degrees pe1,pe2,pe3
%     edge e1: phi_i = lmb_2 * lmb_3^n1,  n1=1,2,...,pe1-1;
%     edge e2: phi_i = lmb_3 * lmb_1^n1,  n1=1,2,...,pe2-1;
%     edge e3: phi_i = lmb_1 * lmb_2^n1,  n1=1,2,...,pe1-1;
% (c) bubble functions - interior polynomial degree == p
%     phi_i = lmb_1 * lmb_2^n1 * lmb_3^n2,
%     for  n1,n2 >= 1  and  n1+n2 <= p-1,
%     i.e.,  pb=3,4,...,p;  n1=1,2,...,pb-2;  n2=pb-1-n1;  <-- this give hierarchic numbering

sft = [2 3 1];

edgefun = [1 1; 1 2; 2 2; 2 3; 3 3; 3 4; 4 4; 4 5; 5 5];

% the number of the shapefunctions
NK = locpdeg(2)+locpdeg(3)+locpdeg(4) + (locpdeg(1)-1)*(locpdeg(1)-2)/2;  % size of AK

% the Peano shape functions (given by exponents k,l,m of the barycentric coordinates)
klm = zeros(NK,3);

% vertex functions
klm(1:3,1:3) = eye(3);  % 3-by-3 identity matrix

nshf = 3;
for edge=1:3
  if (locpdeg(edge+1) > 1)
    edge1 = sft(edge);
    edge2 = sft(edge1);
    for n1=1:locpdeg(edge+1)-1
      nshf = nshf + 1;
      % klm(nshf,1:3) = edgeshfun(edge,i,1:3); --- JEDNA NULA V TROJICI EXPONENTU
      klm(nshf,edge) = 0;
      klm(nshf,edge1) = edgefun(n1, 1); %1;
      klm(nshf,edge2) = edgefun(n1, 2); %n1;

      % to guarantee continuity on edges we have to utilize the global orientation of edges
      % Definition:
      %   The global orientation of an edge v1--v2 agrees with its local orientation if
      %   the global index of the vertex v1 is less than the global index of v2, i.e. if
      %   vgidx(1) < vgidx(2).

      if (gvidx(edge1) > gvidx(edge2))
        % the global and local orientations disagree => interchange the exponents
        klm(nshf,edge1) = edgefun(n1, 2); %n1;
        klm(nshf,edge2) = edgefun(n1, 1); %1;
      end % if

    end % for i
  end % if
end % for edge


% bubble functions
if (locpdeg(1) > 2)
  for pb=3:locpdeg(1)
    for n1=1:pb-2
      n2 = pb-1-n1;
      nshf = nshf + 1;
      klm(nshf,1:3) = [1,n1,n2]; %bubbleshfun(n1,n2,1:3); --- VSECHNY TRI EXPONENTY NENULA
    end % for n2
  end % for n1
end % if

